/*
  project.c - Routines in the Seal PF program for simulating into the future (prediction)
*/

#include "prepro.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <float.h>
#include <limits.h>
#include "global.h"
#include "util.h"
#include "nrutil.h"
#include "numeric.h"
#include "model.h"
#include "filter.h"
#include "project.h"

//This is the only global variable - declared in main.c
extern long glngSeed;

/*------------------------------------------------------------------------*/

void doProjection (controlPFstruct * const pControlPF)
/*
  Samples from a prior particle file and then projects into the future 
  pControlPF->NYears years.  Stores particles as it goes on disk in text
  or binary format.
*/
{

  //Structure to hold the particles
  particlestruct *pParticles;
  //dummy pointer required for SaveParticles
  Int *intDummyIndex;
  Int intYear, intAge, intColony;
  Int i;
  //Temp var to retain whether compact or not
  boolean blnCompact;

  //Set aside memory for the particles
  pParticles = mallocParticleStruct(pControlPF->intNParticles, NAGES,
    NCOLONIES, NPARAMS);
  //and dummy index
  intDummyIndex=ivector(1,pControlPF->intNParticles);

  if (pControlPF->blnDebugFile)
    fprintf(pControlPF->pDebugFile,HEADING(Year 1));

  //Give particles their IDs
  for(i=1;i<=pControlPF->intNParticles;i++) pParticles[i].intID = i;

  //Sample from priors
  readPriorsFromFile(pControlPF, pParticles);

  //Write out Yr1 states to file
  if(pControlPF->intNYears==1) {
    //if it's the last year then only compact if fully compact
    blnCompact=(pControlPF->enumOutputCompact==fullyCompact);
  } else {
    //otherwise compact if !notCompact
    blnCompact=(pControlPF->enumOutputCompact!=notCompact);
  }
  saveParticles(pControlPF, pParticles, intDummyIndex, -1, 1,
    pControlPF->blnOutputBinary, False, False, False, False, 
    False, blnCompact, pControlPF->intNParticles, 
    (pControlPF->blnOutputBinary) ? "": TXTFILE);

  //Loop round years sampling from them and saving to file
  // loop around each remaining year
  for(intYear=2;intYear<=pControlPF->intNYears;intYear++) {

    if (pControlPF->blnDebugFile)
      fprintf(pControlPF->pDebugFile,HEADING(Year %i),intYear);

    //Project forwards
    runPopulationProcess(intYear, True, pControlPF, pParticles);

    //Write out states to file
    if(intYear==pControlPF->intNYears) {
      //if it's the last year then only compact if fully compact
      blnCompact=(pControlPF->enumOutputCompact==fullyCompact);
    } else {
      //otherwise compact if !notCompact
      blnCompact=(pControlPF->enumOutputCompact!=notCompact);
    }
    saveParticles(pControlPF, pParticles, intDummyIndex, -1, intYear,
      pControlPF->blnOutputBinary, True, (intYear==pControlPF->intNYears), 
      False, False, 
      False, blnCompact, pControlPF->intNParticles, 
      (pControlPF->blnOutputBinary) ? "": TXTFILE);

    //Copy new states to old states, ready for next year
    for(i=1;i<=pControlPF->intNParticles;i++) 
      for(intColony=1;intColony<=NCOLONIES;intColony++)
        for(intAge=1;intAge<=NAGES;intAge++)
          pParticles[i].intOldStates[intColony][intAge]=
            pParticles[i].intNewStates[intColony][intAge];

  }  

  //Free memory for the assigned particles
  freeParticleStruct(pParticles, pControlPF->intNParticles, NAGES,
    NCOLONIES, NPARAMS);
}
