#CompareByRegion.r - code to read in regional estimates and plot them
# on the same plots

plot.target<-"png"
filename<-"Age0"
lwd<-2
K<-4
obs.file<-"..//PupData_1984_2014.txt"
obs.file.first.col.year<-T
start.year<-1984
n.state<-1
end.year<-2013

#Read in observed pup values
#----------------------------
#(used to get number of years, number of colonies, etc)
temp <- read.table(obs.file,header=T,na.strings="-1")
if(obs.file.first.col.year) temp<-temp[,-1]
if(length(dim(temp))==1) {
  #If there's only one colony, have to make sure it's an array
  obs.state <-array(temp[1:n.years,],dim=dim(temp))
  dimnames(obs.state)<-dimnames(temp)
  K<-1
} else {
  obs.state<-temp
  K<-dim(obs.state)[2]
}
rm(temp)
#Set start and end year
n.years<-dim(obs.state)[1]
if(is.na(start.year)) {
  start.year<-as.numeric(dimnames(obs.state)[[1]][1])
} else {
  #set the dimnames
  dimnames(obs.state)[[1]]<-start.year:(start.year+n.years-1)
}
if(!is.null(end.year)) {
#  end.year<-which.max(start.year+0:(n.years-1)==end.year)
  n.years<-end.year-start.year+1
} else {
  end.year<-start.year+n.years-1
}
ind<-1:n.years

ylab="Pups"
load("..//WithoutIndependentEst//StatesPupsOldWeights.RData")
data1<-state.estimates
load("..//WithIndependentEst//StatesPupsNewWeights.RData")
data2<-state.estimates

tmp.par<-par(no.readonly=T)

#turn on the printing device
if (plot.target=="pdf") pdf(paste("states",filename,".pdf",sep=""))
if (plot.target=="png") {
  png(paste("states",filename,".png",sep=""),width=1000,height=1000 ,pointsize=18)
  par(lwd=3)
}  
if (plot.target=="bitmap") {
  bitmap(paste("states",filename,".bmp",sep=""),width=7,height=7,units="in",pointsize=10,res=300)#,lwd=2,cex=1.0,cex.axis=1.3,cex.lab=1.3,cex.main=2.0)
  par(lwd=2)
}

if(K>1) par(mfrow=c(2,2),ask=F,cex=1.0,cex.axis=1.3,cex.lab=1.3,cex.main=1.4)
if(K==1) par(mfrow=c(1,1),ask=F)
for(k in 1:K) {
  if (n.state==1) {
    min.y<- min(obs.state[ind,k],data1$lcl.est[ind,k],data2$lcl.est[ind,k],na.rm=T)
    max.y<- max(obs.state[ind,k],data1$ucl.est[ind,k],data2$ucl.est[ind,k],na.rm=T)
    if(sum(is.na(obs.state[ind,k]))<length(obs.state[ind,k])) {
      #at least some non-null pup counts
      plot(dimnames(obs.state)[[1]][ind],obs.state[ind,k],
           xlab="Year",ylab="Pups",type="p",xlim=c(start.year,end.year),ylim=c(min.y,max.y))
      lines((start.year):end.year,data1$mean.est[ind,k],col="blue")
      lines((start.year):end.year,data2$mean.est[ind,k],col="red")
    } else {
      #all observations are null
      plot((start.year):end.year,data1$mean.est[ind,k],type="l",
           xlab="Year",ylab="Pups",xlim=c(start.year,end.year),ylim=c(min.y,max.y),col="blue")
      lines((start.year):end.year,data2$mean.est[ind,k],col="red")
    }
    lines((start.year):end.year,data1$lcl.est[ind,k],lty=3,col="blue")
    lines((start.year):end.year,data1$ucl.est[ind,k],lty=3,col="blue")
    lines((start.year):end.year,data2$lcl.est[ind,k],lty=3,col="red")
    lines((start.year):end.year,data2$ucl.est[ind,k],lty=3,col="red")
  } else {
    min.y<- min(data1$lcl.est[,k],data2$lcl.est[ind,k],na.rm=T)
    max.y<- max(data1$ucl.est[,k],data2$ucl.est[ind,k],na.rm=T)
    plot((start.year):end.year,data1$mean.est[ind,k],type="l",
         xlab="Year",ylab="State",xlim=c(start.year,end.year),ylim=c(min.x,max.x),col="blue")
    lines((start.year):end.year,data1$lcl.est[ind,k],lty=3,col="blue")
    lines((start.year):end.year,data1$ucl.est[ind,k],lty=3,col="blue")
    lines((start.year):end.year,data2$mean.est[ind,k],col="red")
    lines((start.year):end.year,data2$lcl.est[ind,k],lty=3,col="red")
    lines((start.year):end.year,data2$ucl.est[ind,k],lty=3,col="red")
  }
  title(dimnames(obs.state)[[2]][k])
}
#par(mfrow=c(1,1))

#turn off the printing device
if(!(plot.target=="screen")) dev.off()

par(tmp.par)