#Driver file for R post-processing for SCOS 2016

setwd("C:/Users/len/Dropbox/ActiveAnalysis/SCOS16/RCode/WithoutIndependentEst")
#setwd("/data2/len/PostProcess2016/EDDSNM/WithoutIndependentEst")

#density dependent survival model
ddepsurv=TRUE
#whether a fixed sex ratio used or not
fixed.sex.ratio=FALSE
#whether the input files are compact: 0 means compact, 1 means not compact, 2 means compact except last year
# (2 is useful when you want to project forward to future years, in which case you need the last year not
# to be in compact form, but you want space savings on the previous years)
input.files.compact<-2
prefix="../Res"
#Select the plot output type, depending on operating system
plot.target<-ifelse(as.character(Sys.info()["sysname"])=="Windows","png","bitmap")
#set to 0 to have it process the whole files - otherwise choose a low number for a test run
max.particles<-0

source("../CommonRCode/RParams11.r")
param.objects.to.save<-c("id","params","prior.params","prior.params.dist","pweights")
source("../CommonRCode/RStates11.r")

#File of pup production observations
obs.file<-"../PupData_1984_2015.txt"

#What to do
#-----------

do.old.weights<-T
do.new.weights<-F

#Old weights
#-----------

if(do.old.weights){

  #Read parameter values and plot them (also prints out ESS)
  #Note, this function returns the input data straight into the global
  # workspace
  #Note - this also generates the sex ratio file, from the prior which is given in the RParams code file
  RParams(weight.file="Weights",prefix=prefix,input.files.compact=input.files.compact,max.particles=max.particles,
          ddepsurv=ddepsurv,plot.target=plot.target,fixed.sex.ratio=fixed.sex.ratio,generate.sex.ratio.from.prior=TRUE)
  
  #Save the input data as it takes ages to read in, and then delete
  save(list = param.objects.to.save, file = "ParamsOldWeights.RData")
  rm(list=param.objects.to.save)
  
  #Get pup state values and LnL
  state.estimates<-RStates(n.state=1,prefix=prefix,input.files.compact=input.files.compact,max.particles=max.particles,
                           obs.file=obs.file,print.lnl=TRUE,plot.target=plot.target)
  save(state.estimates, file = "StatesPupsOldWeights.RData")
  rm(state.estimates)
  
  #Get adult state values - uses the sex ratio previously generated
  state.estimates<-RStates(n.state= -1,prefix=prefix,input.files.compact=input.files.compact,max.particles=max.particles,
                           obs.file=obs.file,print.lnl=FALSE,plot.target=plot.target)
  save(state.estimates, file = "StatesAdultsOldWeights.RData")
  rm(state.estimates)
}

if(do.new.weights){

  #Get new weight values using the independent total pop estimate, and save to a new weight file
  #---------------
  state.estimates<-RStates(n.state= -1,prefix=prefix,use.add.info.for.weight=TRUE,input.files.compact=input.files.compact,max.particles=max.particles,
                           obs.file=obs.file,print.lnl=FALSE)
  rm(state.estimates)
  
  #New weights
  #-----------
  
  #Get new param values
  RParams(weight.file="AddInfoWeights",prefix=prefix,input.files.compact=input.files.compact,max.particles=max.particles,
          plot.target=plot.target,fixed.sex.ratio=fixed.sex.ratio)
  #Save the input data as it takes ages to read in, and then delete
  save(list = param.objects.to.save, file = "ParamsNewWeights.RData")
  rm(list=param.objects.to.save)
  
  #Get pup state values and LnL
  state.estimates<-RStates(n.state=1,prefix=prefix,input.files.compact=input.files.compact,max.particles=max.particles,
                           obs.file=obs.file,weight.file="AddInfoWeights",print.lnl=TRUE,use.add.info.for.lnl=TRUE,plot.target=plot.target)
  save(state.estimates, file = "StatesPupsNewWeights.RData")
  rm(state.estimates)
  
  #Get adult state values
  state.estimates<-RStates(n.state= -1,prefix=prefix,input.files.compact=input.files.compact,max.particles=max.particles,
                           obs.file=obs.file,weight.file="AddInfoWeights",print.lnl=FALSE,plot.target=plot.target)
  save(state.estimates, file = "StatesAdultsNewWeights.RData")
  rm(state.estimates)

}
