#Updated to have old priors - i.e., from SCOS 2011, with psi estimated from 2014 analysis
#sex.ratio.fixed=F
#read in sex ratio from file and add to the number of parameters - sex ratio is 
# just like another parameter

#RParams.r -- R code to read in files containing parameter values generated 
# by SealPF program, and plot them

#Author: Len Thomas, CREEEM
#Last updated: 20 Aug 2015

RParams<-function(read.results=T,max.particles=0,prefix="NewRes",weight.file="Weights",input.files.compact=T,
                    make.plots=T,plot.target="png",ddepsurv=T,
                    show.sensitivity=F,fixed.sex.ratio=F,generate.sex.ratio.from.prior=F){
#Function to read in parameter data, calculate summary stats and plot
#IMPORTANT NOTE: If parameter data are read in, then the following global vars are created:
# prior.params, prior.params.dist, pweights, id, params
# I do this because the data are very big, and passing them out as a list or some such is not feasible
#Inputs:
# read.results - if T reads in results from file - if F then assumes already read in
#  and are in the workspace (in which case it gets them from there - so violating
#  good programming practice by using data from outside the function)
# max.particles - if zero, reads in all the particles available; otherwise reads at most
#  this many (fewer if fewer are available). Useful for testing the routine before reading in 
#  huge particle files!  Ignored if read.results is F.
#  Note - only works for binary input files
# prefix - if reading in results then the prefix for results files
# weight.file - weight file to read from - either "weights" or "AddInfoWeights"
# input.files.compact - set to T or a positive integer if the input files are compact (i.e., single precision), 
#  F if not
# make plots - set to T if you want plots showing param priors and posteriors
# plot.target - can be screen pdf or png - other options could easily be added
# ddepsurv - set to T (default) for a density dependent survival model, F for dens dep fecundity
#  used for working out carrying capacity priors
# show.sensitivity - set to T if you want the sensitivity statistics
# generate.sex.ratio.file.from.prior - if T then generates a sex.ratio file by sampling from
#  the prior distribution (or the fixed value, if fixed).  Needs to be set to T the first time
#  this function is called after transferring particle data over, so that sex ratio values are
#  generated.
  
  #Additional options on what action is required
  #the proportion of estimates to truncate at either tail of the plots
  truncate <- 0.005
  #Set to T to show the median of the estimates, to T to show the mean
  show.median <- F
  #This routine works by resampling particles, rather than weighted averaging
  # and this is the number to resample
  n.resample<-ifelse(max.particles>0,max.particles,1E6)

  #Options about the data, model, etc
  #----------------------------------
  #Directory to read results files from.  If "" then reads from current directory.
  # if this is a vector of directory paths, then reads from multiple directories
  # and collates the results
  #Note - this  version currently can't cope with multiple directories!
  dir=""
  #Whether input files are text (T) or binary (F)
  input.files.text<-F
  if (input.files.text & max.particles>0) stop("You can only set max.particles for binary input files\n")
  #Whether the input files are in compact format or not - this is
  #  just for the params files, id file, and weights files
  #  If a positive integer is passed in, then 0 means compact, otherwise it's not compact
  if(is.numeric(input.files.compact)) input.files.compact<-(input.files.compact==0)
  if(input.files.compact) {
    size.real<-4 #data saved to binary files as single precision
  } else {
    size.real<-8 #data saved to binary files as double precision
  }
  #number of model parameters - including those fixed, and sex ratio
  n.params<-13
  #number of years of data
  n.years<-32
  #vector of true parameter values - will then plot these as well
  #useful when plotting results from simulation
  truth.vec <- NULL
  #set this to true if the prior was acutally on carrying capacity
  # rather than on beta
  prior.on.cc<-T
  #set to T if dfo model
  dfo<-F
  #set to true to square the value for psi - used in some UK analyses
  sqrpsi<-F
  #File name of the sex ratio file
  sex.ratio.file<-"SexRatio"
  
  #Specifying which priors were used
  #--------------------------------

  #SCOS 2015 analysis priors - ie NEW PRIORS with scaled beta with max 0.97 on adult survival
  #adult survival
  d.Sa <- "scaledbeta"
  Sa <- c(1.6,1.2,0.17, 0.03)
  #max juv survival
  d.Sj.Max <- "beta"
  Sj.Max <- c(2.87,1.78, NA, NA)
  #fecundity
  d.a <- "scaledbeta"
  a <- c(2,1.5,0.4,0)
  #measurement error - fixed at 91.09806 - estimated from a EDDSNM run with prior gamma (2.1,66.67)=mean 140 sd 96.61
  d.psi <- "fixed" #"gamma"
  psi<-c(91.09806, 91.09806, NA, NA)
  #movement params
  d.move.1 <- "fixed"
  d.move.2 <- "fixed"
  d.move.3 <- "fixed"
  move.1.dd <- c(2.25,1.3333,NA, NA)
  move.2.dist <- c(2.25,0.488267,NA, NA)
  move.3.site <-  c(2.25,0.22222,NA, NA)
  #dens dep parameter
  d.ddexp <- "gamma"
  ddexp <- c(4,2.5,NA, NA)
  #carrying capacity parameters
  if(prior.on.cc) {
    d.b<-"gamma"
    b.1<- c(4,2500,NA, NA)
    b.2<- c(4,1250,NA, NA)
    b.3<- c(4,3750,NA, NA)
    b.4<- c(4,10000,NA, NA)
  } else {
    #nothing relevant to dfo analysis here!
    #these are just guesses at this point
  #  d.b<-"gamma"
  #  b.1  <- c(4,0.00027,NA)
  #  b.2  <- c(4,0.000296,NA)
  #  b.3  <- c(4,0.0000740,NA)
  }
  
  #sex ratio prior - gets added to the list of params later
  if(fixed.sex.ratio) {
    d.sex.ratio<-"fixed"
    sex.ratio<-c(1.73,1.73, NA, NA)
  } else {
#    d.sex.ratio<-"uniform"
#    sex.ratio<-c(1,2,NA)
    d.sex.ratio<-"scaledgamma"
    sex.ratio<-c(28.0788, 0.0036998, 1.6, NA) #c(40.350654998, 0.007610101, 1.4) # #c(2,0.1,1)
  }
  
  #make a vector of prior parameters
  prior.params <<- cbind(Sa,Sj.Max,a,
    psi,move.1.dd,move.2.dist,move.3.site,ddexp,b.1,b.2,b.3,b.4,sex.ratio)
  prior.params.dist<<-c(d.Sa,d.Sj.Max,d.a,
     d.psi,d.move.1,d.move.2,d.move.3,d.ddexp,rep(d.b,4),d.sex.ratio)
  if(prior.on.cc) {
    dimnames(prior.params)[[2]]<-c("phi_a","phi_j","alpha_max",
      "psi","gamma_dd","gamma_dist","gamma_sf","rho",
      "chi_ns", "chi_ih", "chi_oh","chi_ork","omega")
  } else {
    #nothing relevant to DFO here!
  }
  names(prior.params.dist) <- dimnames(prior.params)[[2]]

  
  #Read in results
  #----------------------------
  if(read.results) {
    n.dirs<-length(dir)
    if(n.dirs>1) stop("This version can't work with multiple directories")

    #Read in the weights
    weight.file.name<-paste(dir[1],prefix,"Yr",n.years,weight.file,ifelse(input.files.text,".txt",""),sep="")
    if(input.files.text) {
      pweights<<-scan(weight.file.name)
      n.particles<-length(pweights)
    } else {
      size<-file.info(weight.file.name)$size
      n.particles<-size/size.real
      #set n.particles to max.particles if necessary
      if((max.particles>0)&(n.particles>max.particles)) n.particles<-max.particles
      #open and read in
      weights.file<-file(description=weight.file.name,open="rb")
      pweights<<-readBin(weights.file,"numeric",size=size.real,n=n.particles)
      close(weights.file)
    }

    #Read in the IDs and params
    id.file.name <- paste(dir[1],prefix,"Yr",n.years,"ID",ifelse(input.files.text,".txt",""),sep="")
    param.file.name <- paste(dir[1],prefix,"Yr",n.years,"Params",ifelse(input.files.text,".txt",""),sep="")
    if(input.files.text) {
      id<<-scan(id.file.name)
      params.tmp<-matrix(scan(param.file.name),n.params-1,length(pweights),byrow=FALSE)
    } else {
      id.file<-file(description=id.file.name,open="rb")
      id<<-readBin(id.file,"integer",n=n.particles)
      close(id.file)
      param.file<-file(description=param.file.name,open="rb")
      params.tmp<-matrix(readBin(param.file,"numeric",size=size.real,n=n.particles*(n.params-1)),n.params-1,n.particles,byrow=FALSE)
      close(param.file)
    }

    #read in or simulate sex ratio, and add to params matrix
    sex.ratio.file.name<-paste(dir[1],prefix,"Yr",n.years,sex.ratio.file,ifelse(input.files.text,".txt",""),sep="")
    if(generate.sex.ratio.from.prior){

      #Simulate sex ratios
      if(d.sex.ratio=="fixed"){
        #generate from fixed value
        psexratio<-rep(sex.ratio[1],n.particles)
      } else {
        if(d.sex.ratio=="scaledgamma"){
          #generate from scaled gamma
          psexratio<- sex.ratio[3]+rgamma(n.particles,shape=sex.ratio[1],scale=sex.ratio[2])
        } else {
          if(d.sex.ratio=="uniform"){
            psexratio<-runif(n.particles,sex.ratio[1],sex.ratio[2])
          } else {
            stop("Sex ratio prior not from a known distribution.")
          }
        }
      }
      #save to the sex ratio file
      if(input.files.text) {
        cat(psexratio,file=sex.ratio.file.name,sep="\n",append=F)
      } else {
        #write out
        sex.ratio.file<-file(description=sex.ratio.file.name,open="wb")
        writeBin(psexratio,sex.ratio.file,size=size.real)
        close(sex.ratio.file)
      }

    } else {

      #read in
      if(input.files.text) {
        psexratio<-scan(sex.ratio.file.name)
      } else {
        size<-file.info(sex.ratio.file.name)$size
        n.particles<-size/size.real
        #open and read in
        sex.ratio.file<-file(description=sex.ratio.file.name,open="rb")
        psexratio<-readBin(sex.ratio.file,"numeric",size=size.real,n=n.particles)
        close(sex.ratio.file)
      }

    }
    
    #create params maxtrix
    
    params<<-rbind(params.tmp,psexratio)
    dimnames(params)<-list(names(prior.params.dist),NULL)
    rm(params.tmp,psexratio)
    gc()
    
    if(sqrpsi) {
      params["psi",]<-params["psi",]^2
    }

    #Function to return the fecundity given age
    #used in getting carrying capacity
    get.a<-function(amax,rho,age) {
      if(age<=3) return(0)
      if(age>=6) return(amax)
      temp = exp(rho*(age-4.5))
      temp = temp/(1+temp)
      a<-amax*temp
      return(a)
    }

    #Function to return carrying capacity parameter for a given set of parameter values
    get.dds.cc<-function(beta,amax,Sj,Sa,rho,ddepsurv){
      if(ddepsurv) {
        if(dfo) {
          lambda2<-0.5*Sa^3*
            (get.a(amax,rho,4) + Sa*get.a(amax,rho,5)  +
             Sa^2*get.a(amax,rho,6)/(1-Sa))
          cc<-1/beta*(Sj*lambda2 - 1)
        } else {
          cc<-(( amax*0.5*Sj*Sa^5/(1-Sa) - 1)^(1/rho))/beta
        }
      } else {
        if (dfo) {
          stop("Program currently not set up to work out cc under dens dep fecundity for DFO analysis")
        } else {
          #dfo model not currently set up for this
          tau<-(0.5*Sj*Sa^5)/(1-Sa)
          cc<-((amax*tau - 1) ^ (1/rho)) / (beta*tau)
        }
      }
      return(cc)
    }


    if(prior.on.cc) {
      #if I forgot to set a fixed prior on rho in the ControlPF then set it here, otherwise it screws
      # up the cc stuff
      if(d.ddexp == "fixed") params["rho",]<-ddexp[1]
      #need to transform the cc parameter read in into a carrying capacity
      params["chi_ns",]<-get.dds.cc(params["chi_ns",],params["alpha_max",],params["phi_j",],params["phi_a",],params["rho",],ddepsurv)
      params["chi_ih",]<-get.dds.cc(params["chi_ih",],params["alpha_max",],params["phi_j",],params["phi_a",],params["rho",],ddepsurv)
      params["chi_oh",]<-get.dds.cc(params["chi_oh",],params["alpha_max",],params["phi_j",],params["phi_a",],params["rho",],ddepsurv)
      params["chi_ork",]<-get.dds.cc(params["chi_ork",],params["alpha_max",],params["phi_j",],params["phi_a",],params["rho",],ddepsurv)
    }

    #Function to work out effective sample size
    ess<-function(x) {
      m<-mean(x)
      v<-var(x)
      return(length(x)/(1+v/(m^2)))
    }

    ess.unique<-function(pweights,ids) {
    #Works out the ESS of unique particles, summing weights for each unique particle

      #work out sum of weights for each unique particle
      unique.particle.weights<-as.vector(by(pweights,ids,sum))
      ess<-ess(unique.particle.weights)
      return(list(unique=length(unique.particle.weights),ess=ess))
    }

    cat(paste("ESS:", ess(pweights), "\n"))
    ess.unique<-ess.unique(pweights,id)
    cat(paste("Total particles:", length(id), " Unique particles:", ess.unique$unique, " ESS_unique:", ess.unique$ess, "\n"))


  } else {
    #dont' read in results
    cat("Results not being read from file\n")
  }

  #Do some processing after reading in the data
  #--------------------------------------------

  #Sample from the parameters with prob proportional to the pweights
  j<-sample(1:length(pweights),n.resample,replace=T,prob=pweights)
  params2<-params[,j]
  dimnames(params2)<-list(names(prior.params.dist), NULL)

  #Get number of fixed parameters
  param.fixed <- (prior.params.dist=="fixed")
  N.nonfixed.params <- n.params - sum(param.fixed)

  #Define makebreaks function - makes the histogram breaks
  makebreaks <- function(numbreaks=10,data=NULL,truth=NULL)
  #returns an array of break points, given a vecor of data, and the true value
  {
     max.data <- max(data)
     min.data <- min(data)
     if (!is.null(truth)) {
       if (truth > max.data) max.data <- truth
       if (truth < min.data) min.data <- truth
     }
     breaks <- seq(min.data,max.data,length=numbreaks)
     return(breaks)
  }

  dscaledbeta<-function(x, shape1, shape2, ncp = 0, log = FALSE, mix.prop = 1, mix.prop2=0){
  #Purpose: Just like the beta density, except it's defined as having density 0 from
  #  x=0 to x=(1-mix.prop-mix.prop2), between ((1-mix.prop-mix.prop2)>x>(1-mix.prop2)) 
  #  having the ordinary beta density
  #  but with x scaled according to (x-(1-mix.prop-mix.prop2))/(mix.prop)

    y<-(x-(1-mix.prop-mix.prop2))/mix.prop
    res<-rep(ifelse(log,-Inf,0),length(x))
    ind<-(y>0)
    #need to divide by mix.prop to rescale so it integrates to 1
    res[ind]<-dbeta(y[ind],shape1,shape2,ncp,log)/mix.prop
    return(res)
  }

  dscaledgamma<-function(x, shape, rate=1, scale=1/rate, log = FALSE, shift =0){
    #Purpose: Just like thegamma density, except it's defined as having density 0 from
    #  x=0 to x=shift, and then after that (x>shift) having the ordinary gamma density
    
    y<-x-shift
    res<-rep(ifelse(log,-Inf,0),length(x))
    ind<-(y>0)
    
    #Set the scale regardless of whether rate or scale were passed in
    if(scale!=1/rate){
      #if rate !=1 then rate must have been passed in, so work out what that means for scale
      if(rate!=1) scale=1/rate
    }
    
    #don't worry about resaling as the shift is small compared with overall density
    res[ind]<-dgamma(y[ind],shape,scale=scale,log=log)
    return(res)
  }

  #Do the plotting
  #---------------
  if (make.plots) {

    #turn on the printing device
    if (plot.target=="pdf") pdf(paste("params",weight.file,".pdf",sep=""))
    if (plot.target=="png") png(paste("params",weight.file,".png",sep=""),width=1000,height=1000,pointsize=26)#,cex=1.0,cex.axis=1.3,cex.lab=1.3,cex.main=2.0)
    if (plot.target=="bitmap") bitmap(paste("params",weight.file,".bmp",sep=""),width=7,height=7,units="in",pointsize=13,res=300)#,cex=1.0,cex.axis=1.3,cex.lab=1.3,cex.main=2.0)

    #set layout of plots - number of plots per row and column
    cols<- floor(sqrt(N.nonfixed.params))
    par(mfrow=c(cols,ceiling(N.nonfixed.params/cols)))

    #Do the plotting
    for (i in 1:n.params) {
      if( !(param.fixed[i]))  {
        #remove the NAs and Infs
        param.temp<-params2[i,is.finite(params2[i,])]
        #Work out mean, sd, etc
        mean.param <- ifelse(show.median,median(param.temp),mean(param.temp))
        sd.param <- sd(param.temp)
        cv.param <- sd.param*100/mean.param
        truth<-truth.vec[i]
        #get the upper and lower bounds on what to plot
        if (truncate>0) {
          upper<-quantile(param.temp,1-truncate)
          #use truth if that is higher
          if(!is.null(truth)) { if (truth>upper) upper<-truth}
          usedata <- param.temp[param.temp<upper]
          lower<-quantile(param.temp,truncate)
          #use truth if that is lower
          if(!is.null(truth)) { if (truth<lower) lower<-truth}
          usedata <- usedata[usedata>lower]
        } else {
          usedata <- param.temp
        }
        #Set the breaks
        breaks <- makebreaks(data=usedata,truth=truth)
        #Show the histogram
        hist(usedata,main=paste(dimnames(params2)[[1]][i],"\n",signif(mean.param,3),
          " (", signif(sd.param,3), ")" ,sep=""),breaks=breaks,
          xlab="",probability=T)
        abline(v=mean.param, lty=3, col=5,lwd=2)
        if(!is.null(truth.vec)) abline(v=truth,lty=1,col=3,lwd=2)
        #Draw the prior on top
        x <- seq(min(breaks),max(breaks),length=1000)
        prior <- switch(prior.params.dist[i],
          uniform=dunif(x,prior.params[1,i],prior.params[2,i]),
          gamma=dgamma(x,prior.params[1,i],scale=prior.params[2,i]),
          beta=dbeta(x,prior.params[1,i],prior.params[2,i]),
          scaledbeta=dscaledbeta(x,prior.params[1,i],prior.params[2,i],mix.prop=prior.params[3,i],mix.prop2=prior.params[4,i]),
          scaledgamma=dscaledgamma(x,shape=prior.params[1,i],scale=prior.params[2,i],shift=prior.params[3,i]))
        lines(x,prior,type="l",lwd=2,col=6)
      }
    }

    par(mfrow=c(1,1))

    #turn off the printing device
    if(!(plot.target=="screen")) dev.off()
  }

  #Calculate the sensitivity stats
  #-------------------------------
  if(show.sensitivity) {
    ratio<-matrix(NA, n.params,2)
    dimnames(ratio) [[2]] <- c("Ratio1","Ratio2")
    dimnames(ratio) [[1]] <- dimnames(prior.params)[[2]]
    for (i in 1:n.params) {
      if( !(param.fixed[i]|any(is.nan(params2[i,]))) ) {
        if(prior.params.dist[i]=="uniform") {
          #do nothing - can't deal with this one!
        }
        if(prior.params.dist[i]=="gamma") {
          ratio[i,1] <- cov(params2[i,],log(params2[i,])) / prior.params[2,i]
          ratio[i,2] <- var(params2[i,])/(prior.params[1,i]*prior.params[2,i]^2)
        }
        if(prior.params.dist[i]=="beta") {
          ratio[i,1] <- cov(params2[i,],log(params2[i,])) /
            (prior.params[2,i]/(prior.params[1,i]+prior.params[2,i])^2)
          ratio[i,2] <- cov(params2[i,],log((1-params2[i,]))) /
            (-prior.params[1,i]/(prior.params[1,i]+prior.params[2,i])^2)
        }
      }
    }
    print(ratio)
  }

  
  invisible(NULL)
}